"use strict";

exports.__esModule = true;
exports.createInstance = undefined;

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var isFunction = function isFunction(arg) {
  return typeof arg === "function";
};

/**
 * createInstance allows you to create instances of Async that are bound to a specific promise.
 * A unique instance also uses its own React context for better nesting capability.
 */
var createInstance = exports.createInstance = function createInstance() {
  var defaultProps = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _React$createContext = _react2.default.createContext(),
      Consumer = _React$createContext.Consumer,
      Provider = _React$createContext.Provider;

  var Async = function (_React$Component) {
    _inherits(Async, _React$Component);

    function Async(props) {
      _classCallCheck(this, Async);

      var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

      _initialiseProps.call(_this);

      var promiseFn = props.promiseFn || defaultProps.promiseFn;
      var initialValue = props.initialValue || defaultProps.initialValue;
      var initialError = initialValue instanceof Error ? initialValue : undefined;
      var initialData = initialError ? undefined : initialValue;

      _this.mounted = false;
      _this.counter = 0;
      _this.args = [];
      _this.state = {
        initialValue: initialValue,
        data: initialData,
        error: initialError,
        isLoading: !initialValue && isFunction(promiseFn),
        startedAt: undefined,
        finishedAt: initialValue ? new Date() : undefined,
        cancel: _this.cancel,
        run: _this.run,
        reload: function reload() {
          _this.load();
          _this.run.apply(_this, _this.args);
        },
        setData: _this.setData,
        setError: _this.setError
      };
      return _this;
    }

    Async.prototype.componentDidMount = function componentDidMount() {
      this.mounted = true;
      this.state.initialValue || this.load();
    };

    Async.prototype.componentDidUpdate = function componentDidUpdate(prevProps) {
      if (prevProps.watch !== this.props.watch) this.load();
      if (prevProps.promiseFn !== this.props.promiseFn) {
        this.cancel();
        if (this.props.promiseFn) this.load();
      }
    };

    Async.prototype.componentWillUnmount = function componentWillUnmount() {
      this.cancel();
      this.mounted = false;
    };

    Async.prototype.render = function render() {
      var children = this.props.children;

      if (isFunction(children)) {
        return _react2.default.createElement(
          Provider,
          { value: this.state },
          children(this.state)
        );
      }
      if (children !== undefined && children !== null) {
        return _react2.default.createElement(
          Provider,
          { value: this.state },
          children
        );
      }
      return null;
    };

    return Async;
  }(_react2.default.Component);

  /**
   * Renders only when deferred promise is pending (not yet run).
   *
   * @prop {boolean} persist Show until we have data, even while loading or when an error occurred
   * @prop {Function|Node} children Function (passing state) or React node
   */


  var _initialiseProps = function _initialiseProps() {
    var _this2 = this;

    this.load = function () {
      var promiseFn = _this2.props.promiseFn || defaultProps.promiseFn;
      if (!promiseFn) return;
      _this2.counter++;
      _this2.setState({ isLoading: true, startedAt: new Date(), finishedAt: undefined });
      return promiseFn(_this2.props).then(_this2.onResolve(_this2.counter), _this2.onReject(_this2.counter));
    };

    this.run = function () {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var deferFn = _this2.props.deferFn || defaultProps.deferFn;
      if (!deferFn) return;
      _this2.counter++;
      _this2.args = args;
      _this2.setState({ isLoading: true, startedAt: new Date(), finishedAt: undefined });
      return deferFn.apply(undefined, args).then(_this2.onResolve(_this2.counter), _this2.onReject(_this2.counter));
    };

    this.cancel = function () {
      _this2.counter++;
      _this2.setState({ isLoading: false, startedAt: undefined });
    };

    this.onResolve = function (counter) {
      return function (data) {
        if (_this2.mounted && _this2.counter === counter) {
          var onResolve = _this2.props.onResolve || defaultProps.onResolve;
          _this2.setData(data, function () {
            return onResolve && onResolve(data);
          });
        }
        return data;
      };
    };

    this.onReject = function (counter) {
      return function (error) {
        if (_this2.mounted && _this2.counter === counter) {
          var onReject = _this2.props.onReject || defaultProps.onReject;
          _this2.setError(error, function () {
            return onReject && onReject(error);
          });
        }
        return error;
      };
    };

    this.setData = function (data, callback) {
      _this2.setState({ data: data, error: undefined, isLoading: false, finishedAt: new Date() }, callback);
      return data;
    };

    this.setError = function (error, callback) {
      _this2.setState({ error: error, isLoading: false, finishedAt: new Date() }, callback);
      return error;
    };
  };

  Async.Pending = function (_ref) {
    var children = _ref.children,
        persist = _ref.persist;
    return _react2.default.createElement(
      Consumer,
      null,
      function (state) {
        if (state.data !== undefined) return null;
        if (!persist && state.isLoading) return null;
        if (!persist && state.error !== undefined) return null;
        return isFunction(children) ? children(state) : children || null;
      }
    );
  };

  /**
   * Renders only while loading.
   *
   * @prop {boolean} initial Show only on initial load (data is undefined)
   * @prop {Function|Node} children Function (passing state) or React node
   */
  Async.Loading = function (_ref2) {
    var children = _ref2.children,
        initial = _ref2.initial;
    return _react2.default.createElement(
      Consumer,
      null,
      function (state) {
        if (!state.isLoading) return null;
        if (initial && state.data !== undefined) return null;
        return isFunction(children) ? children(state) : children || null;
      }
    );
  };

  /**
   * Renders only when promise is resolved.
   *
   * @prop {boolean} persist Show old data while loading
   * @prop {Function|Node} children Function (passing data and state) or React node
   */
  Async.Resolved = function (_ref3) {
    var children = _ref3.children,
        persist = _ref3.persist;
    return _react2.default.createElement(
      Consumer,
      null,
      function (state) {
        if (state.data === undefined) return null;
        if (!persist && state.isLoading) return null;
        if (!persist && state.error !== undefined) return null;
        return isFunction(children) ? children(state.data, state) : children || null;
      }
    );
  };

  /**
   * Renders only when promise is rejected.
   *
   * @prop {boolean} persist Show old error while loading
   * @prop {Function|Node} children Function (passing error and state) or React node
   */
  Async.Rejected = function (_ref4) {
    var children = _ref4.children,
        persist = _ref4.persist;
    return _react2.default.createElement(
      Consumer,
      null,
      function (state) {
        if (state.error === undefined) return null;
        if (state.isLoading && !persist) return null;
        return isFunction(children) ? children(state.error, state) : children || null;
      }
    );
  };

  return Async;
};

exports.default = createInstance();